;;
;; Elisp scripts for generating GObjects and C-Objects
;; Author: Tomasz Gebarowski <gebarowski@gmail.com>
;;
;; Last update: Sat Feb 05 13:35:58 2011
;;
;; In order to make it working with your version of Emacs, follow the steps:
;;
;;    * Copy azt-code.el into ~/.emacs.d
;;    * Create new directory for templates ~/.emacs.d/azt-code and put all 
;;      the object templates from objects directory.
;;    * Edit your ~/.emacs and put the following line:
;;
;;      (load-file "~/.emacs.d/azt-code.el")
;;
;; When you reload your .emacs or restart the whole Emacs,
;; you will have two new commands available:
;;
;; azt-c-object-create - Creating C-Object from the template
;; azt-gobject-create - Creating GObject from the template
;;
;;

;; User-defined Settings
(setq azt_template_author "Tomasz Gebarowski <gebarowski@gmail.com>")
(setq azt_template_copyright "Put your copyright or whatever else here")

;; Template Constants
;; Set Azetti Code Template Directory
(setq azt_template_dir (concatenate 'string user-emacs-directory "azt-code/"))
;; GObject string constants
(setq azt_gobject_template_file "azt_code_simple_gobject")
(setq azt_gobject_template_class "AztCodeSimpleGObject")
(setq azt_gobject_template_macro "AZT_CODE_SIMPLE_GOBJECT")
(setq azt_gobject_template_doxygroup "AztCodeSimpleGObjectGObject")
;; C-Object string constants
(setq azt_object_template_file "azt_code_simple_object")
(setq azt_object_template_class "AztCodeSimpleObject")
(setq azt_object_template_macro "AZT_CODE_SIMPLE_OBJECT")
(setq azt_object_template_doxygroup "AztCodeSimpleObjectObject")

;; Delimeter defnitions
(setq azt_template_descr_tag "<<DESCRIPTION>>")
(setq azt_template_author_tag "<<AUTHOR>>")
(setq azt_template_date_tag "<<DATE>>")
(setq azt_template_copyright_tag "<<COPYRIGHT>>")


;;
;; Sample EmacsLisp code to un-CamelCase a string (from http://www.emacswiki.org/emacs/CamelCase)
;;
(defun un-camelcase-string (s &optional sep start)
"Convert CamelCase string S to lower case with word separator SEP.
 Default for SEP is a hyphen \"-\".

 If third argument START is non-nil, convert words after that
 index in STRING."
    (let ((case-fold-search nil))
        (while (string-match "[A-Z]" s (or start 1))
              (setq s (replace-match (concat (or sep "-") 
                                      (downcase (match-string 0 s))) 
                                      t nil s)))
    (downcase s)))


(defun azt-gobject-create (gobject_name gobject_path gobject_description) 
    "Create Azetti GObject interactively from Azt-Code template"
    (interactive "sNew GObject name: 
DGObject target path:
sGObject short description:")

    ;;Initiate file names
    (setq file_basename (un-camelcase-string gobject_name "_"))

    (setq ext '("h" "c"))
    (dolist (item ext)
        (setq src (format "%s/%s.%s" azt_template_dir azt_gobject_template_file item))
        (setq dst (format "%s/%s.%s" gobject_path file_basename item))
        ;; Copy file from template to dest name
        (copy-file src dst)
        ;; Open newly created file in new buffer
        (find-file dst)
        ;; Replace strings from template
        (replace-string azt_gobject_template_doxygroup (format "%s%s" gobject_name "GObject"))
        (beginning-of-buffer)
        (beginning-of-buffer)
        (replace-string azt_gobject_template_file file_basename) 
        (beginning-of-buffer)
        (replace-string azt_gobject_template_class gobject_name)
        (beginning-of-buffer)
        (replace-string azt_gobject_template_macro (upcase file_basename)) 
        (beginning-of-buffer)
        (replace-string azt_template_descr_tag gobject_description) 
        (beginning-of-buffer)
        (replace-string azt_template_author_tag azt_template_author) 
        (beginning-of-buffer)
        (replace-string azt_template_date_tag (format-time-string "%a %b %d %H:%M:%S %Y")) 
        (beginning-of-buffer)
        (replace-string azt_template_copyright_tag azt_template_copyright) 
        (beginning-of-buffer)
        (save-buffer)
    )
)

(defun azt-c-object-create (object_name object_path object_description) 
    "Create Azetti C-Object interactively from Azt-Code template"
    (interactive "sNew C-Object name: 
DC-Object target path:
sC-Object short description:")

    ;;Initiate file names
    (setq file_basename (un-camelcase-string object_name "_"))

    (setq ext '("h" "c"))
    (dolist (item ext)
        (setq src (format "%s/%s.%s" azt_template_dir azt_object_template_file item))
        (setq dst (format "%s/%s.%s" object_path file_basename item))
        ;; Copy file from template to dest name
        (copy-file src dst)
        ;; Open newly created file in new buffer
        (find-file dst)
        ;; Replace strings from template
        (replace-string azt_object_template_doxygroup (format "%s%s" object_name "Object"))
        (beginning-of-buffer)
        (beginning-of-buffer)
        (replace-string azt_object_template_file file_basename) 
        (beginning-of-buffer)
        (replace-string azt_object_template_class object_name)
        (beginning-of-buffer)
        (replace-string azt_object_template_macro (upcase file_basename)) 
        (beginning-of-buffer)
        (replace-string azt_template_descr_tag object_description) 
        (beginning-of-buffer)
        (replace-string azt_template_author_tag azt_template_author) 
        (beginning-of-buffer)
        (replace-string azt_template_date_tag (format-time-string "%a %b %d %H:%M:%S %Y")) 
        (beginning-of-buffer)
        (replace-string azt_template_copyright_tag azt_template_copyright) 
        (beginning-of-buffer)
        (save-buffer)
    )
)
